# -*- coding: utf-8 -*-
def display_as_tree(lst, i=0, indent_root="", indent_others="    "):
    """Display a list as a binary tree.

    :param lst: A list.
    :param i: An integer. The index of the root node to consider.
    :param indent_root: A string to be printed before the root.
    :param indent_others: A string to be printed before its children.

    The list lst is considered as a binary tree, where the children of
    element lst[k] are lst[2 * k + 1] and lst[2 * k + 2] (if they exist).
    """
    if i >= len(lst):
        return
    print(indent_root + "|-> " + str(lst[i]))
    display_as_tree(lst, 2 * i + 1, indent_root=indent_others,
                    indent_others=indent_others + "|   ")
    display_as_tree(lst, 2 * i + 2, indent_root=indent_others,
                    indent_others=indent_others + "    ")


def siftdown(lst, i):
    """Siftdown.

    :param lst: A numerical list. The children trees of position i in lst
    must be heaps.
    :param i: An integer (index in the lst).

    This function modifies lst *in place*. If necessary, it recursively sifts
    down element lst[i] so that, in the end, the tree rooted at position i
    in lst is a heap.
    """
    l = 2 * i + 1
    r = 2 * i + 2
    # Where is the maximal element: in i or one of its two children?
    maxi = i
    if l < len(lst) and lst[l] > lst[maxi]:
        maxi = l
    if r < len(lst) and lst[r] > lst[maxi]:
        maxi = r
    # If it is not in i, we exchange and continue siftdown.
    if maxi != i:
        lst[i], lst[maxi] = lst[maxi], lst[i]
        print(lst)
        display_as_tree(lst)
        siftdown(lst, maxi)


def heapify(lst):
    """Heapify a list

    :param lst: A numerical list.

    This function modifies list lst *in place*, so that lst becomes a heap.
    """
    for i in range(len(lst) // 2 - 1, -1, -1):
        print("Siftdown element " + str(lst[i]) + ":")
        siftdown(lst, i)


def heap_sort(lst):
    """Sort a list using heap sort.

    :param lst: A numerical list.

    This function sorts list lst *in place*.
    """
    print("")
    print("*************")
    print("*  Heapify  *")
    print("*************")
    heapify(lst)

    print("")
    print("**********")
    print("*  Sort  *")
    print("**********")
    result = []
    while lst:
        print("Exchange:")
        lst[-1], lst[0] = lst[0], lst[-1]
        print(lst)
        display_as_tree(lst)
        print("Transfer:")
        result.insert(0, lst.pop())
        print("result =", result)
        print(lst)
        display_as_tree(lst)
        if lst:
            print("Siftdown element " + str(lst[0]) + ":")
        siftdown(lst, 0)
    lst.extend(result)


if __name__ == "__main__":

    print("Exercise 1")
    print("----------")
    A = ['x', 1, 'x', 9, 7, 'x', 'x', 6, 8, 2]
    print(A)
    display_as_tree(A)

    print("Siftdown element " + str(A[1]) + ":")
    siftdown(A, 1)

    print("")
    print("Exercises 2 and 3")
    print("----------------")

    A = [4, 1, 3, 8, 2, 0, 5, 6, 9, 7]
    print(A)
    display_as_tree(A)

    heap_sort(A)

    print("\nFinal result:")
    print(A)
